unit Unit2;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VCLTee.TeEngine, Vcl.ExtCtrls,
  VCLTee.TeeProcs, VCLTee.Chart, VCLTee.Series, Generics.Collections,
  Vcl.StdCtrls;

type
  TChartSeriesValues = array of currency;

  TChartSeriesItem = record
    Name: string;
    Color: TColor;
    Values: TChartSeriesValues;
  end;

  TChartSeriesItems = array of TChartSeriesItem;

const
  CHART_DATA: array[0..3] of TChartSeriesItem =
    ((Name: 'Social Security'; Color: clGreen;   Values: [3734.10,3846.12,3961.50,4080.35,4202.76,4328.84,4458.71,4592.47,4730.24,4872.15,5018.31,5168.86,5323.93,5483.65,5648.16,5817.60,5992.13,6171.89,6357.05,6547.76,6744.19,6946.52,7154.92,7369.57]),
     (Name: 'Pre-Tax';         Color: clBlue;    Values: [4280.89,4309.14,4438.42,4571.57,4708.60,4849.93,4995.47,5145.34,5299.62,5458.69,5622.17,5790.87,5964.25,6143.08,6327.33,4822.78,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00]),
     (Name: 'Roth';            Color: clPurple;  Values: [0.00,0.00,0.00,0.00,0.11,0.05,0.00,0.00,0.08,0.00,0.29,0.26,0.61,0.72,0.79,1.01,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00]),
     (Name: 'Shortfall';       Color: clRed;     Values: [0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,0.00,1694.17,6713.50,6914.91,7122.35,7336.03,7556.11,7782.79,8016.26,7987.99]));


type
  TChartStepDown = class
  private
    FSeriesList: TChartSeriesList;
    FX: Integer;
    FValue: double;
  public
    constructor Create(const aX: integer; const aValue: double);
    destructor Destroy; override;
    property X: Integer read FX;
    property Value: double read FValue;
    property SeriesList: TChartSeriesList read FSeriesList;
  end;

  TChartStepDowns = class(TObjectList<TChartStepDown>)
  public
    function Add(const aX: integer; const aValue: double; aSeries: TChartSeries): TChartStepDown; overload;
  end;


  TForm2 = class(TForm)
    Chart1: TChart;
    Series1: TAreaSeries;
    Panel1: TPanel;
    Button1: TButton;
    chkIncludeDecimals: TCheckBox;
    procedure Button1Click(Sender: TObject);
  private
    procedure UpdateChart(aChart: TChart);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form2: TForm2;

implementation


{$R *.dfm}

{ TChartStepDown }

constructor TChartStepDown.Create(const aX: integer; const aValue: double);
begin
  FSeriesList := TChartSeriesList.Create;
  FX := aX;
  FValue := aValue;
end;

destructor TChartStepDown.Destroy;
begin
  FSeriesList.Free;
  inherited;
end;

{ TChartStepDowns }

function TChartStepDowns.Add(const aX: integer;
  const aValue: double; aSeries: TChartSeries): TChartStepDown;
var
  I: Integer;
begin
  result := nil;
  for I := 0 to Count - 1 do
    if Items[I].X = aX then begin
      result := Items[I];
      break;
    end;

  if not assigned(result) then begin
    result := TChartStepDown.Create(aX, aValue);
    Add(result);
  end;

  result.SeriesList.Add(aSeries);
end;


{ TForm2 }

procedure TForm2.Button1Click(Sender: TObject);
begin
  UpdateChart(Chart1);
end;

procedure TForm2.UpdateChart(aChart: TChart);


  function CreateChartSeries(aSeriesData: TChartSeriesItem): TAreaSeries;
  begin
    result := TAreaSeries.Create(self);
    result.ParentChart := aChart;
    result.AreaLinesPen.Visible := false;
    result.Pen.Visible := false;

    if aChart.SeriesCount > 1 then
      aChart.SeriesList.Move(aChart.SeriesCount - 1, 0);{}

    result.Title := aSeriesData.Name;
    result.Color := aSeriesData.Color;
  end;
var
  lLastIdx: integer;
  lChartStepDowns: TChartStepDowns;
  lSeriesData: TChartSeriesItem;
  lSrs: TChartSeries;
  I, J: integer;
  lIncome: currency;
  lHorzCaption: string;
  lMonthlyIncomes: array of currency;
  lChartStepDown: TChartStepDown;
begin
  lLastIdx := Length(CHART_DATA[0].Values)-1;
  for I := 1 to High(CHART_DATA) do
    Assert(lLastIdx + 1 = Length(CHART_DATA[I].Values));

  SetLength(lMonthlyIncomes, lLastIdx + 1);
  for I := 0 to Length(lMonthlyIncomes)-1 do
      lMonthlyIncomes[I] := 0;

  Chart1.FreeAllSeries;

  lChartStepDowns := TChartStepDowns.Create;
  try
    for I := 0 to High(CHART_DATA) do begin
     lSeriesData := CHART_DATA[I];

     lsrs := CreateChartSeries(lSeriesData);

      for J := 0 to lLastIdx do begin
        lIncome := lSeriesData.Values[J];
        if not chkIncludeDecimals.Checked then
          lIncome := Trunc(lIncome);

        if (I = 0) then
          lHorzCaption :=  ' '
        else
          lHorzCaption := inttostr(I);

        //see if previous month income was zero, if so, add 0 here
        if (lIncome > 0) then begin
          lMonthlyIncomes[J] := lMonthlyIncomes[J] + lIncome;
          lsrs.AddXY(J, lMonthlyIncomes[J], lHorzCaption);
          if (J < lLastIdx) and (lSeriesData.Values[J + 1] = 0) then
            lChartStepDowns.Add(J, (lSeriesData.Values[J] - lIncome), lsrs);
        end else if lMonthlyIncomes[I] > 0 then
          lsrs.AddXY(j, lMonthlyIncomes[J], lHorzCaption)
        else begin
          if (I = 3) and (I < lLastIdx) and  (lSeriesData.Values[I + 1] > 0) then
            lsrs.AddXY(J, lMonthlyIncomes[J], lHorzCaption)
          else
            lsrs.AddNullXY(J, 0, lHorzCaption);
        end;
      end;
    end;

    for I := lChartStepDowns.Count-1 downto 0 do begin
      lChartStepDown := lChartStepDowns[I];

      for J := 0 to aChart.SeriesCount - 1 do begin
        lSrs  := aChart.Series[J];
        if lChartStepDown.SeriesList.IndexOf(lSrs) > -1 then
          lSrs.AddXY(lChartStepDown.X, lChartStepDown.Value)
        else
          lSrs.AddXY(lChartStepDown.X, lSrs.YValue[lChartStepDown.X]);
      end;
    end;
  finally // wrap up
    lChartStepDowns.Free;
  end;    // try/finally
end;


end.
